#!/bin/python3
#%%
import os

#%%
import pandas as pd
from sklearn.linear_model import LogisticRegression
import xgboost
from xgboost import XGBClassifier
from lightgbm import LGBMClassifier

#%%
from tools.std_lib.basic import find_matching_paths
from tools.data_processing.easy_pandas import pickle_load, json_load

#%%

class ModelUtility(object):

    def __init__(self, clf_path: str):
        self.clf_path = clf_path
        clf_info_path = find_matching_paths(
            clf_path, 
            pattern='clf.json', 
            expected_count=1, 
            search_type='file'
            )[0]
        clf_dir = os.path.dirname(clf_info_path)

        self.dir = clf_dir

        self.clf_info = json_load(clf_info_path)
        self.prefix = self.clf_info['main']['prefix']
        self.exp_name = self.clf_info['main']['exp_name']
        self.save_name = self.clf_info['main']['save_name']

        pickle_path = os.path.join(clf_dir, 'clf_pickle')
        self.clf = pickle_load(pickle_path)

        keys_path = os.path.join(clf_dir, 'keys.json')
        self.features = json_load(keys_path)

        fillna_path = os.path.join(clf_dir, 'fillna.json')
        self.fillna = json_load(fillna_path)

        return None
    

    def get_features(self, return_type = 'df'):
        features = pd.Series(self.features)
        features = pd.Series(features, name = 'feature')
        features.index = pd.Index(range(1, len(features)+1), name = 'index')
        if return_type == 'df':
            result = features.to_frame('feature')
        elif return_type == 's':
            result = features
        return result
    
        
    def get_feature_importances(
            self, 
            return_type = 'series', 
            sort_by_importance = False, 
            positive_only = False,
            n_top = None,
            ):

        clf_info = self.clf_info
        clf = self.clf

        if isinstance(clf, LogisticRegression):
            stat_df = pd.DataFrame(
                clf_info['more']['feature']['feature_importances']['LogisticRegression']['feature_importances_stat']
                )
            result = stat_df.set_index('feature')['feature_importances']
        
        elif isinstance(clf, XGBClassifier) or isinstance(clf, LGBMClassifier):
            origin_importance_type = clf.importance_type
            clf.importance_type = 'gain'
            features = self.features
            feature_importances = clf.feature_importances_
            
            result = pd.Series(dict(zip(
                features, feature_importances
                )), dtype=float)
            result = result / result.sum()
            clf.importance_type = origin_importance_type
            
        elif isinstance(clf, XGBClassifier):
            pass

        elif isinstance(clf, LGBMClassifier):
            pass

        if sort_by_importance:
            result.sort_values(ascending=False, inplace=True)

        if sort_by_importance and n_top is not None:
            if isinstance(n_top, int) and n_top > 0:
                result = result.iloc[:n_top]
            if isinstance(n_top, float) and n_top < 1.0:
                cumsum = result.cumsum()
                idx = cumsum[cumsum > n_top].index[0]
                result = result.loc[:idx] #.sum()
            elif isinstance(n_top, float) and abs(n_top - 1) < 1e-6:
                result = result[result > 0] #.sum()

        if positive_only:
            result = result[result > 0]

        if return_type == 'dict':
            result = result.to_dict()
        elif return_type in ['df', 'dataframe']:
            result = result.to_frame()
            result.index.name = 'feature'
            result.columns = ['importance']

        print('feature_importances.sum():\n', result.sum())
        
        return result
    

    def dump_clf(self, model_format: str):
        if isinstance(self.clf, XGBClassifier) and model_format == 'ubj':
            save_path = os.path.join(self.dir, 'XGBClassifier.ubj')
            self.clf.save_model(save_path)
        return save_path
    

    def load_clf(self, model_format: str):
        if isinstance(self.clf, XGBClassifier) and model_format == 'ubj':
            save_path = os.path.join(self.dir, 'XGBClassifier.ubj')
            ubj = xgboost.Booster()
            ubj.load_model(save_path)
            self.ubj = ubj
        return self.ubj
    

    def predict_proba(self, 
                      X_df: pd.DataFrame, 
                      return_type = 'df',
                      name = 'prefix'
                      ):
        if name == 'prefix':
            name = self.prefix
        elif name == 'exp_name':
            name = self.exp_name
        X_df = X_df[self.features].fillna(self.fillna)
        result = self.clf.predict_proba(X_df)[:, 1]
        result = pd.Series(result, name=name, index=X_df.index)
        if return_type == 'df':
            result = result.to_frame()
        return result
    

    def get_path(self, key: str):
        if key == 'feature':
            path = os.path.join(self.clf_path, 'keys.json')
        elif key == 'ubj':
            path = os.path.join(self.clf_path, 'XGBClassifier.ubj')
        return path
    
    
    def alter_predict(self, 
                      model_format: str,
                      X_df: pd.DataFrame, 
                      return_type = 'df',
                      name = 'prefix'
                      ):
        
        if name == 'prefix':
            name = self.prefix
        elif name == 'exp_name':
            name = self.exp_name
        X_df = X_df[self.features].fillna(self.fillna)
        index = X_df.index

        if model_format == 'ubj':
            data_DM = xgboost.DMatrix(X_df)
            clf_ubj = self.load_clf(model_format='ubj')
            result = clf_ubj.predict(data_DM)

        result = pd.Series(result, name = name, index = index)
        if return_type == 'df':
            result = result.to_frame()

        return result




    def predict_proba_by_pmml(self, X_df, fillna = True):
        pass

    def add_pmml_files(self):
        pass

    def save(self, dir_path: str):
        pass


    # end of class



# path = './merchants/m37/p15_modeling/p15_mdls-new2/p15_com_mdls/p15-com/p15-com-LR_cw1mi1000-dim=S01_bp+S02+S03+S09_dim300/p15-com-LR_cw1mi1000-dim=S01_bp+S02+S03+S09_dim300_rs0-ds0-20250212Wed183957/'
# path = './merchants/m37/p15_modeling/p15_mdls-new2/p15_src_mdls/p15-S13-jtstyf_rundb'

# clf = ModelPackage(path)

# d = json_load(path)

# %%
